(() => {
  var __commonJS = (cb, mod) => function __require() {
    return mod || (0, cb[Object.keys(cb)[0]])((mod = { exports: {} }).exports, mod), mod.exports;
  };

  // src/camel_to_dash.js
  var require_camel_to_dash = __commonJS({
    "src/camel_to_dash.js"(exports, module) {
      function camelToDash(key) {
        return key.replace(/([A-Z])/g, "-$1").toLowerCase();
      }
      module.exports = camelToDash;
    }
  });

  // src/constants.js
  var require_constants = __commonJS({
    "src/constants.js"(exports, module) {
      module.exports = {
        CLASS_NAME: "className",
        DEFAULT_NODE_NAME: "node",
        EMPTY_ATTRS: Object.freeze({}),
        NOM_ATTR_KEY: "data-nom-key",
        NOM_HANDLERS_KEY: "data-nomhandlers",
        PROP_ATTRS: ["checked", "value"],
        STYLE_NODE_NAME: "style"
      };
    }
  });

  // src/element.js
  var require_element = __commonJS({
    "src/element.js"(exports, module) {
      var camelToDash = require_camel_to_dash();
      var constants = require_constants();
      var Element = class {
        constructor(nodeName, args, parent, optNamespace) {
          this.nodeName = nodeName || constants.DEFAULT_NODE_NAME;
          this.domElement = null;
          this.attrs = args && args.attrs || constants.EMPTY_ATTRS;
          this.parent = parent;
          this.namespace = optNamespace;
          this._children = null;
          this._textContent = null;
          this._unsafeContent = args && args.unsafeContent;
          this._textValue = this.attrs && this.attrs.textValue || null;
          this.childNodes = [];
          this.isCollapsible = false;
          this.selectors = null;
          this.keyframes = null;
          this.render = null;
          this.hasUpdateableHandler = false;
          if (args) {
            if (args.inlineTextChild) {
              this.childNodes.push(new Element("text", { attrs: { textValue: args.inlineTextChild } }, this));
              this._textValue = args.inlineTextChild;
            }
          }
        }
        addSelector(selector, rules) {
          if (this.nodeName !== constants.STYLE_NODE_NAME) {
            throw new Error("Selectors can only be added to style nodes");
          }
          if (this.selectors === null) {
            this.selectors = [];
          }
          this.selectors.push({
            selector,
            rules
          });
        }
        renderSelectorRules(entries, rules) {
          Object.keys(rules).forEach((key) => {
            entries.push(`${camelToDash(key)}:${rules[key]};`);
          });
        }
        renderSelectors() {
          const entries = [];
          this.selectors.forEach((selector) => {
            const name = selector.selector;
            entries.push(`${name}{`);
            const rules = selector.rules;
            if (name.indexOf("@") === 0) {
              Object.keys(rules).forEach((key) => {
                entries.push(`${key} {`);
                this.renderSelectorRules(entries, rules[key]);
                entries.push(`} `);
              });
            } else {
              this.renderSelectorRules(entries, selector.rules);
            }
            entries.push("}");
          });
          return entries.join("");
        }
        replaceChild(newChild, oldChild) {
          const kids = this.childNodes;
          for (var i = 0, len = kids.length; i < len; ++i) {
            if (kids[i] === oldChild) {
              kids[i].parent = null;
              kids[i] = newChild;
              break;
            }
          }
        }
        getAttribute(key) {
          return this.attrs && this.attrs[key];
        }
        get id() {
          return this.attrs && this.attrs.id;
          ;
        }
        get className() {
          return this.attrs && this.attrs.className;
        }
        get textValue() {
          return this._textValue;
        }
        get unsafeContent() {
          return this._unsafeContent;
        }
        get textContent() {
          if (!this._textContent) {
            const str = this.childNodes.map((node) => node.textContent || node.textValue).join("");
            this._textContent = str !== "" ? str : null;
          }
          return this._textContent || "";
        }
        get children() {
          if (!this._children) {
            this._children = this.childNodes.filter((node) => node.nodeName !== "text");
          }
          return this._children;
        }
        get firstChild() {
          return this.childNodes && this.childNodes[0] || null;
        }
      };
      module.exports = Element;
    }
  });

  // src/scheduler.js
  var require_scheduler = __commonJS({
    "src/scheduler.js"(exports, module) {
      function scheduler(externalNextFrame) {
        const pending = [];
        let responseIsPending = false;
        function onNextFrame(executeHandler) {
          if (!responseIsPending) {
            responseIsPending = true;
            externalNextFrame(() => {
              responseIsPending = false;
              executeHandler();
            });
          }
        }
        function execute() {
          const filtered = [];
          const pendingCopy = pending.slice();
          pending.length = 0;
          function filterChildren(entries) {
            return entries.filter((entry, index) => {
              let parent = entry.element.parent;
              while (parent) {
                if (elementIsPending(entries, parent)) {
                  filtered.push(entry);
                  return false;
                }
                parent = parent.parent;
              }
              return true;
            });
          }
          filterChildren(pendingCopy).forEach((entry) => {
            entry.handler();
          });
          filtered.forEach((entry) => {
            if (entry.handler.onSkipped) {
              entry.handler.onSkipped();
            }
          });
          if (pending.length > 0) {
            onNextFrame(execute);
          }
        }
        function schedule(element, handler) {
          pending.push({ element, handler });
          onNextFrame(execute);
        }
        schedule.forceUpdate = execute;
        return schedule;
      }
      function elementIsPending(entries, nomElement) {
        return entries.some((entry) => {
          return entry.element === nomElement;
        });
      }
      module.exports = scheduler;
    }
  });

  // src/config.js
  var require_config = __commonJS({
    "src/config.js"(exports, module) {
      var scheduler = require_scheduler();
      var globalKeys;
      function getOrCreateGlobalKeys() {
        if (!globalKeys) {
          globalKeys = {
            builderStack: null,
            schedule: null,
            setTimeout: null,
            requestAnimationFrame: null
          };
          globalKeys.builderStack = [];
          globalKeys.setTimeout = global.setTimeout.bind(global);
          globalKeys.requestAnimationFrame = (global.requestAnimationFrame || globalKeys.setTimeout).bind(global);
          globalKeys.schedule = scheduler(globalKeys.requestAnimationFrame);
        }
        return globalKeys;
      }
      function config(optKeys) {
        const existingKeys = getOrCreateGlobalKeys();
        if (optKeys) {
          Object.assign(existingKeys, optKeys);
        }
        return existingKeys;
      }
      config.reset = function() {
        globalKeys = null;
      };
      module.exports = config;
    }
  });

  // src/builder.js
  var require_builder = __commonJS({
    "src/builder.js"(exports, module) {
      var Element = require_element();
      var camelToDash = require_camel_to_dash();
      var config = require_config();
      function top() {
        const stack = config().builderStack;
        return stack[stack.length - 1] || null;
      }
      function processClassName(value) {
        if (Array.isArray(value)) {
          return value.filter((entry) => !!entry && entry !== -1).join(" ");
        } else if (value && typeof value === "object") {
          if (value._isUnsafe) {
            return value;
          } else {
            return Object.keys(value).filter((key) => !!value[key] && value[key] !== -1).join(" ");
          }
        }
        return value;
      }
      function getUpdateScheduler(elem, handler) {
        return function _getUpdateScheduler(optCompleteHandler) {
          if (elem.hasUpdateableHandler) {
            let renderNow2 = function() {
              elem.render(builder, handler, optCompleteHandler);
            };
            var renderNow = renderNow2;
            if (optCompleteHandler) {
              renderNow2.onSkipped = function() {
                optCompleteHandler(elem);
              };
            }
            config().schedule(elem, renderNow2);
          }
        };
      }
      function processHandler(elem, handler) {
        const stack = config().builderStack;
        stack.push(elem);
        if (handler.length > 0) {
          elem.hasUpdateableHandler = true;
          handler(getUpdateScheduler(elem, handler));
        } else {
          handler();
        }
        stack.pop();
      }
      function processStyleObject(obj) {
        const parts = [];
        Object.keys(obj).forEach((key) => {
          const value = obj[key];
          if (value !== null && value !== "" && value !== false) {
            parts.push(`${camelToDash(key)}:${obj[key]};`);
          }
        });
        return parts.join("");
      }
      function processAttrs(attrs) {
        if (attrs) {
          if (attrs.className) {
            attrs.className = processClassName(attrs.className);
          } else if (attrs.style && typeof attrs.style === "object") {
            attrs.style = processStyleObject(attrs.style);
            if (attrs.style === "") {
              delete attrs.style;
            }
          } else if (attrs.onenter && attrs.onkeyup || attrs.onEnter && attrs.onKeyUp) {
            throw new Error("onenter and onkeyup are mutually exclusive, consider adding a switch statement to your onkeyup handler for the enter case.");
          }
        }
        return attrs;
      }
      function processArgs(...args) {
        let attrs;
        let handler;
        let inlineTextChild;
        let unsafeContent;
        let type;
        args.forEach((value) => {
          type = typeof value;
          if (type !== "undefined") {
            if (type === "number" || type === "string") {
              inlineTextChild = value;
            } else if (value && type === "object" && value._isUnsafe) {
              unsafeContent = value.content;
            } else if (type === "function") {
              handler = value;
            } else if (value) {
              attrs = processAttrs(value);
            }
          }
        });
        return {
          attrs,
          inlineTextChild,
          unsafeContent,
          handler
        };
      }
      function builder(nodeName, optAttrs, optHandler, optContent, optNamespace) {
        try {
          const parent = top();
          const attrs = processArgs(optAttrs, optHandler, optContent);
          const elem = new Element(nodeName, attrs, parent, optNamespace);
          if (parent) {
            parent.childNodes.push(elem);
          }
          if (attrs.handler) {
            processHandler(elem, attrs.handler);
          }
          return elem;
        } catch (err) {
          const stack = config().builderStack;
          stack.length = 0;
          console.error(err);
          throw err;
        }
      }
      builder.forceUpdate = function _forceUpdate() {
        config().schedule.forceUpdate();
      };
      builder.externalElement = function(external) {
        const elem = builder("external");
        elem.domElement = external;
        return elem;
      };
      builder.elementWrapper = function(nodeName, optNamespace) {
        return function _elementWrapper(optAttrs, optHandler, optContent) {
          return builder(nodeName, optAttrs, optHandler, optContent, optNamespace);
        };
      };
      builder.addSelector = function(selector, styles) {
        top().addSelector(selector, styles);
      };
      builder.addKeyframe = function(name, rules) {
        top().addSelector(`@keyframes ${name}`, rules);
      };
      builder.unsafeContent = function(content) {
        return {
          _isUnsafe: true,
          content
        };
      };
      module.exports = builder;
    }
  });

  // src/dom.js
  var require_dom = __commonJS({
    "src/dom.js"(exports, module) {
      var builder = require_builder();
      var dom = {};
      dom.htmlFourNodes = [
        "a",
        "abbr",
        "acronym",
        "address",
        "applet",
        "area",
        "b",
        "base",
        "basefont",
        "bdo",
        "big",
        "blockquote",
        "body",
        "br",
        "button",
        "caption",
        "center",
        "cite",
        "code",
        "col",
        "colgroup",
        "dd",
        "del",
        "dfn",
        "dir",
        "div",
        "dl",
        "dt",
        "em",
        "fieldset",
        "font",
        "form",
        "frame",
        "frameset",
        "h1",
        "h2",
        "h3",
        "h4",
        "h5",
        "h6",
        "head",
        "hr",
        "html",
        "i",
        "iframe",
        "img",
        "input",
        "ins",
        "isindex",
        "kbd",
        "label",
        "lengend",
        "li",
        "link",
        "map",
        "menu",
        "meta",
        "noframes",
        "noscript",
        "object",
        "ol",
        "optgroup",
        "option",
        "p",
        "param",
        "pre",
        "q",
        "s",
        "samp",
        "script",
        "select",
        "small",
        "span",
        "strike",
        "strong",
        "style",
        "sub",
        "sup",
        "table",
        "tbody",
        "td",
        "textarea",
        "tfoot",
        "th",
        "thead",
        "title",
        "tr",
        "tt",
        "tu",
        "u",
        "ul",
        "var",
        "xmp"
      ];
      dom.htmlFiveNodes = dom.htmlFourNodes.concat([
        "article",
        "aside",
        "command",
        "details",
        "summary",
        "figure",
        "figcaption",
        "footer",
        "header",
        "hgroup",
        "mark",
        "meter",
        "nav",
        "progress",
        "ruby",
        "rt",
        "rp",
        "section",
        "time",
        "wbr",
        "audio",
        "video",
        "source",
        "embed",
        "canvas",
        "datalist",
        "keygen",
        "output",
        "tel",
        "search",
        "url",
        "email",
        "datetime",
        "date",
        "month",
        "week",
        "datetime-local",
        "number",
        "ranger",
        "color"
      ]);
      dom.collapsibleNodes = [
        "br",
        "hr",
        "img"
      ];
      dom.htmlFiveNodes.forEach((nodeName) => {
        dom[nodeName] = builder.elementWrapper(nodeName);
      });
      dom.collapsibleNodes.forEach((nodeName) => {
        dom[nodeName] = builder.elementWrapper(nodeName);
      });
      dom.text = builder.elementWrapper("text");
      dom.stylesheet = (href) => {
        return dom.link({ type: "text/css", rel: "stylesheet", href });
      };
      dom.external = builder.externalElement;
      dom.selector = builder.addSelector;
      dom.keyframes = builder.addKeyframe;
      dom.unsafe = builder.unsafeContent;
      module.exports = dom;
    }
  });

  // src/events.js
  var require_events = __commonJS({
    "src/events.js"(exports, module) {
      var events = {
        KEY_DOWN: "keydown",
        KEY_UP: "keyup",
        ON_ENTER: "onenter",
        ON_KEY_DOWN: "onkeydown",
        ON_KEY_UP: "onkeyup"
      };
      module.exports = events;
    }
  });

  // src/generate_id.js
  var require_generate_id = __commonJS({
    "src/generate_id.js"(exports, module) {
      var lut = [];
      for (let i = 0; i < 256; i += 1) {
        lut[i] = (i < 16 ? "0" : "") + i.toString(16);
      }
      function generateId() {
        const d0 = Math.random() * 4294967295 | 0;
        const d1 = Math.random() * 4294967295 | 0;
        const id = lut[d0 & 255] + lut[d0 >> 8 & 255] + lut[d0 >> 16 & 255] + lut[d0 >> 24 & 255] + "-" + lut[d1 & 255] + lut[d1 >> 8 & 255];
        return id;
      }
      module.exports = generateId;
    }
  });

  // src/html_encode.js
  var require_html_encode = __commonJS({
    "src/html_encode.js"(exports, module) {
      var sharedTextArea = null;
      function getTextArea(document) {
        if (!sharedTextArea) {
          sharedTextArea = document.createElement("textarea");
        }
        return sharedTextArea;
      }
      function htmlEncode(str, optDocument) {
        const type = typeof str;
        if (type === "object" && str && str._isUnsafe && str.content) {
          return str.content;
        }
        if (type === "number" || type === "boolean") {
          return String(str);
        } else if (str === null || type === "undefined" || !str.replace) {
          return "";
        }
        if (optDocument) {
          const textArea = getTextArea(optDocument);
          textArea.textContent = str;
          return textArea.innerHTML;
        } else {
          var map = {
            "&": "&amp;",
            "<": "&lt;",
            ">": "&gt;",
            '"': "&quot;",
            "'": "&#39;"
          };
          return str.replace(/[&<>"']/g, function(m) {
            return map[m];
          });
        }
      }
      module.exports = htmlEncode;
    }
  });

  // src/operations.js
  var require_operations = __commonJS({
    "src/operations.js"(exports, module) {
      var config = require_config();
      var constants = require_constants();
      var htmlEncode = require_html_encode();
      function top(stack) {
        return stack[stack.length - 1];
      }
      function setId(value) {
        return function _setId(domElement, stack, document) {
          domElement.id = value;
          return domElement;
        };
      }
      function setAttribute(name, value) {
        return function _setAttribute(domElement, stack, document) {
          const updatedName = name === "key" ? constants.NOM_ATTR_KEY : name;
          if (typeof value === "boolean" && !value) {
            domElement.removeAttribute(updatedName);
          } else {
            domElement.setAttribute(updatedName, name === "href" ? value : htmlEncode(value));
          }
          if (constants.PROP_ATTRS.indexOf(name) > -1) {
            domElement[name] = value;
          }
          return domElement;
        };
      }
      function setDataAttribute(name, value) {
        return function _setDataAttribute(domElement, stack, document) {
          const updated = typeof value === "string" ? htmlEncode(value, document) : value;
          domElement.dataset[name] = updated;
          return domElement;
        };
      }
      function removeAttribute(name) {
        return function _removeAttribute(domElement) {
          if (constants.PROP_ATTRS.indexOf(name) > -1) {
            domElement[name] = "";
          }
          domElement.removeAttribute(name);
          return domElement;
        };
      }
      function _setRenderFunction(getUpdateElement, nomElement, document, domElement) {
        if (nomElement.hasUpdateableHandler) {
          nomElement.render = getUpdateElement(nomElement, document, domElement);
        }
      }
      function enqueueOnRender(handler) {
        return function _enqueueOnRender(domElement) {
          return function _enqueueOnRenderInternal() {
            try {
              handler(domElement);
            } catch (err) {
              console.error(err);
            }
          };
        };
      }
      function setClassName(value) {
        return function _setClassName(domElement, stack, document) {
          domElement.className = htmlEncode(value, document);
          return domElement;
        };
      }
      function removeClassName() {
        return function _removeClassName(domElement) {
          domElement.removeAttribute("class");
          return domElement;
        };
      }
      function setHandler(key, value) {
        return function _setHandler(domElement) {
          domElement[key] = value;
          const handlersString = domElement.getAttribute(constants.NOM_HANDLERS_KEY);
          const handlers = handlersString ? handlersString.split(" ") : [];
          if (handlers.indexOf(key) === -1) {
            handlers.push(key);
          }
          domElement.setAttribute(constants.NOM_HANDLERS_KEY, handlers.join(" "));
          return domElement;
        };
      }
      function removeHandler(key) {
        return function _removeHandler(domElement) {
          domElement[key] = null;
          const handlers = domElement.getAttribute(constants.NOM_HANDLERS_KEY).split(" ");
          const index = handlers.indexOf(key);
          if (index > -1) {
            handlers.splice(index, 1);
          }
          if (handlers.length === 0) {
            domElement.removeAttribute(constants.NOM_HANDLERS_KEY);
          } else {
            domElement.setAttribute(constants.NOM_HANDLERS_KEY, handlers.join(" "));
          }
          return domElement;
        };
      }
      function pushElement(nomElement) {
        return function _pushElement(domElement, stack) {
          stack.push(domElement);
          return domElement;
        };
      }
      function pushDomElement(nomElement, getUpdateElement, domElement) {
        return function _pushDomElement(_domElement, stack, document) {
          _setRenderFunction(getUpdateElement, nomElement, document, domElement);
          return domElement;
        };
      }
      function popElement() {
        return function _popElement(domElement, stack) {
          return stack.pop();
        };
      }
      function createElement(nomElement, getUpdateElement) {
        return function _createElement(domElement, stack, document) {
          let newDomElement;
          if (nomElement.namespace) {
            newDomElement = document.createElementNS(nomElement.namespace, nomElement.nodeName);
          } else {
            newDomElement = document.createElement(nomElement.nodeName);
          }
          _setRenderFunction(getUpdateElement, nomElement, document, newDomElement);
          return newDomElement;
        };
      }
      function createTextNode(content) {
        return function _createTextNode(domElement, stack, document) {
          const text = document.createTextNode(htmlEncode(content, document));
          const parent = top(stack);
          parent.appendChild(text);
          return parent;
        };
      }
      function updateInnerHTML(content) {
        return function _updateInnerHTML(domElement, stack, document) {
          domElement.innerHTML = content;
          return domElement;
        };
      }
      function updateTextContent(content) {
        return function _updateTextContent(domElement, stack, document) {
          domElement.textContent = htmlEncode(content, document);
          return domElement;
        };
      }
      function createExternalElement(nomElement) {
        return function _createExternalElement(domElement, stack) {
          const parent = top(stack);
          const dElem = nomElement.domElement;
          if (parent) {
            parent.appendChild(dElem);
          }
          return dElem;
        };
      }
      function appendChild() {
        return function _appendChild(domElement, stack) {
          const parent = top(stack);
          if (parent) {
            parent.appendChild(domElement || optDomElement);
          }
          return domElement;
        };
      }
      function moveToIndex(existingNode, index) {
        return function _insertDomBefore(domElement, stack, document) {
          const parent = top(stack);
          if (parent) {
            const sibling = parent.childNodes[index];
            if (sibling !== domElement) {
              parent.insertBefore(existingNode, sibling);
            }
          }
        };
      }
      function removeChild(child) {
        return function _removeChild(domElement, stack) {
          top(stack).removeChild(child);
          return domElement;
        };
      }
      function removeAllChildren() {
        return function _removeAllChildren(domElement, stack) {
          while (domElement.firstChild) {
            domElement.removeChild(domElement.firstChild);
          }
          return domElement;
        };
      }
      function replaceChild(existingElement) {
        return function _replaceChild(domElement, stack) {
          top(stack).replaceChild(domElement, existingElement);
          return domElement;
        };
      }
      function execute(ops, doc, domElement) {
        const outerStack = [];
        const trailingActions = [];
        const root = ops.reduce((lastDomElement, operation) => {
          const result = operation(lastDomElement, outerStack, doc);
          if (typeof result === "function") {
            trailingActions.push(result);
            return lastDomElement;
          }
          return result;
        }, domElement);
        config().setTimeout(() => {
          trailingActions.forEach((action) => {
            action();
          });
        });
        return root;
      }
      module.exports = {
        appendChild,
        createElement,
        createExternalElement,
        createTextNode,
        enqueueOnRender,
        execute,
        moveToIndex,
        popElement,
        pushDomElement,
        pushElement,
        removeAllChildren,
        removeAttribute,
        removeChild,
        removeClassName,
        removeHandler,
        replaceChild,
        setAttribute,
        setClassName,
        setDataAttribute,
        setHandler,
        setId,
        updateInnerHTML,
        updateTextContent
      };
    }
  });

  // src/ready.js
  var require_ready = __commonJS({
    "src/ready.js"(exports, module) {
      var DOM_CONTENT_LOADED = "DOMContentLoaded";
      function ready(document, callback) {
        function handler(event) {
          callback();
          document.removeEventListener(DOM_CONTENT_LOADED, handler);
        }
        document.addEventListener(DOM_CONTENT_LOADED, handler);
      }
      module.exports = ready;
    }
  });

  // src/render_element.js
  var require_render_element = __commonJS({
    "src/render_element.js"(exports, module) {
      var constants = require_constants();
      var events = require_events();
      var operations = require_operations();
      function getUpdateElement(nomElement, doc, lastDomElement) {
        return function _updateElement(builder, handler, optCompleteHandler) {
          const newNomElement = builder(nomElement.nodeName, nomElement.attrs, handler);
          newNomElement.render = nomElement.render;
          if (nomElement.parent) {
            nomElement.parent.replaceChild(newNomElement, nomElement);
          }
          const newDomElement = renderElement(newNomElement, doc, lastDomElement);
          if (lastDomElement !== newDomElement && lastDomElement.parentNode) {
            lastDomElement.parentNode.replaceChild(newDomElement, lastDomElement);
          }
          if (optCompleteHandler && typeof optCompleteHandler === "function") {
            setTimeout(() => {
              optCompleteHandler(newDomElement, newNomElement);
            });
          }
        };
      }
      function nomElementChildNodesToOperations(ops, nomElement, doc, optDomElement2) {
        getChildUpdateStrategy(nomElement, optDomElement2)(ops, nomElement, doc, optDomElement2);
      }
      function updateOrCreateEachChildStrategy(ops, nomElement, doc, optDomElement2) {
        const kids = nomElement.childNodes || [];
        for (let i = 0, len = kids.length; i < len; ++i) {
          const nomChild = kids[i];
          const domChild = optDomElement2 && optDomElement2.childNodes[i];
          nomElementToOperations(ops, nomChild, doc, domChild);
        }
      }
      function replaceAllChildrenStrategy(ops, nomElement, doc, domElement) {
        while (domElement.firstChild) {
          domElement.removeChild(domElement.firstChild);
        }
        updateOrCreateEachChildStrategy(ops, nomElement, doc, domElement);
      }
      function updateChildrenWithKeysStrategy(ops, nomElement, doc, domElement) {
        const domKids = Array.prototype.slice.call(domElement.children);
        const nomKids = nomElement.children;
        const mapped = [];
        for (var i = 0, len = nomKids.length; i < len; ++i) {
          const nomKid = nomKids[i];
          const key = nomKid.getAttribute("key");
          const matched = removeChildWithMatchingKey(key, domKids);
          if (matched) {
            mapped.push({
              nom: nomKid,
              dom: matched
            });
            ops.push(operations.moveToIndex(matched, i));
          }
        }
        while (domKids.length > 0) {
          ops.push(operations.removeChild(domKids.shift()));
        }
      }
      function removeChildWithMatchingKey(keyValue, domElements) {
        for (var i = 0, len = domElements.length; i < len; ++i) {
          const domElement = domElements[i];
          if (getNomKeyValue(domElement) === keyValue) {
            domElements.splice(i, 1);
            return domElement;
          }
        }
        return null;
      }
      function getChildUpdateStrategy(nomElement, optDomElement2) {
        if (optDomElement2 && allChildrenHaveKeys(nomElement, optDomElement2)) {
          return updateChildrenWithKeysStrategy;
        } else if (!optDomElement2 || childrenArInterchangeable(nomElement, optDomElement2)) {
          return updateOrCreateEachChildStrategy;
        } else {
          return replaceAllChildrenStrategy;
        }
      }
      function elementsMatch(domKids) {
        return function(child, index) {
          return child.nodeName === domKids[index].nodeName.toLowerCase();
        };
      }
      function childrenArInterchangeable(nomElement, domElement) {
        return nomElement.childNodes.length === domElement.childNodes.length && nomElement.childNodes.every(elementsMatch(domElement.childNodes));
      }
      function allChildrenHaveKeys(nomElement, optDomElement2) {
        if (!optDomElement2 || optDomElement2.children.length === 0) {
          return false;
        }
        const kids = optDomElement2.children;
        for (var i = 0, len = kids.length; i < len; ++i) {
          if (!getNomKeyValue(kids[i])) {
            return false;
          }
        }
        return true;
      }
      function getNomKeyValue(optDomElement2) {
        return optDomElement2 && optDomElement2.getAttribute(constants.NOM_ATTR_KEY);
      }
      function createOnEnterHandler(handler) {
        return function(event) {
          if (event && event.keyCode === 13) {
            handler(event);
          }
        };
      }
      function createDispatcherOperation(ops, nomElement, name, handler) {
        if (name === events.ON_ENTER) {
          ops.push(operations.setHandler(events.ON_KEY_UP, createOnEnterHandler(handler)));
        } else {
          ops.push(operations.setHandler(name, handler));
        }
      }
      function getAllAttributeKeys(attrs, optDomElement2) {
        const allKeys = Object.keys(attrs || constants.EMPTY_ATTRS);
        if (optDomElement2) {
          const domKeys = Array.prototype.slice.call(optDomElement2.attributes).map((attr) => {
            return attr.name === "class" ? "className" : attr.name;
          });
          return allKeys.concat(domKeys);
        }
        return allKeys;
      }
      function isEmptyObject(value) {
        return value && typeof value === "object" && Object.keys(value).length === 0;
      }
      function elementAttributesToOperations(ops, nomElement, optDomElement2) {
        const attrs = nomElement.attrs || constants.EMPTY_ATTRS;
        getAllAttributeKeys(attrs, optDomElement2).forEach((keyWithCase) => {
          const value = attrs[keyWithCase];
          const lowerCaseKey = keyWithCase.toLowerCase();
          if (lowerCaseKey === "id") {
            if (!optDomElement2 || value !== optDomElement2.id) {
              ops.push(operations.setId(value));
            }
          } else if (lowerCaseKey === "classname") {
            if (optDomElement2 && value !== optDomElement2.className) {
              if (!value) {
                ops.push(operations.removeClassName());
              } else {
                ops.push(operations.setClassName(value));
              }
            } else {
              ops.push(operations.setClassName(value));
            }
          } else if (lowerCaseKey === "key") {
            if (!optDomElement2 || value !== getNomKeyValue(optDomElement2)) {
              ops.push(operations.setAttribute(lowerCaseKey, value));
            }
          } else if (lowerCaseKey === "onrender") {
            if (typeof value === "function") {
              ops.push(operations.enqueueOnRender(value));
            } else {
              throw new Error("onRender requires a function handler");
            }
          } else if (typeof value === "function") {
            createDispatcherOperation(ops, nomElement, lowerCaseKey, value);
          } else if (!optDomElement2) {
            if (!isEmptyObject(value)) {
              ops.push(operations.setAttribute(keyWithCase, value));
            }
          } else if (optDomElement2) {
            const domValue = optDomElement2.getAttribute(keyWithCase);
            if (value === "" || value === void 0 || value === null) {
              ops.push(operations.removeAttribute(keyWithCase, value));
            } else if (value != domValue) {
              ops.push(operations.setAttribute(keyWithCase, value));
            }
          }
        });
      }
      function nomElementToOperations(ops, nomElement, doc, optDomElement2) {
        if (nomElement.nodeName === "text") {
          ops.push(operations.removeAllChildren());
          ops.push(operations.createTextNode(nomElement.textValue));
        } else if (nomElement.nodeName === "external") {
          ops.push(operations.createExternalElement(nomElement));
        } else {
          if (!optDomElement2) {
            ops.push(operations.createElement(nomElement, getUpdateElement));
          } else {
            ops.push(operations.pushDomElement(nomElement, getUpdateElement, optDomElement2));
          }
          ops.push(operations.pushElement(nomElement));
          if (nomElement.nodeName === constants.STYLE_NODE_NAME && nomElement.selectors) {
            ops.push(operations.updateInnerHTML(nomElement.renderSelectors()));
          }
          elementAttributesToOperations(ops, nomElement, optDomElement2);
          if (nomElement.unsafeContent) {
            ops.push(operations.removeAllChildren());
            ops.push(operations.updateInnerHTML(nomElement.unsafeContent));
          } else {
            nomElementChildNodesToOperations(ops, nomElement, doc, optDomElement2);
          }
          ops.push(operations.popElement());
          if (!optDomElement2) {
            ops.push(operations.appendChild());
          }
        }
      }
      function renderElement(nomElement, doc, optDomElement2) {
        if (!doc) {
          throw new Error("renderElement requires a reference to an HTML document.");
        }
        const ops = [];
        nomElementToOperations(ops, nomElement, doc, optDomElement2);
        return operations.execute(ops, doc, optDomElement2);
      }
      module.exports = renderElement;
    }
  });

  // src/render_string.js
  var require_render_string = __commonJS({
    "src/render_string.js"(exports, module) {
      var camelToDash = require_camel_to_dash();
      var constants = require_constants();
      var dom = require_dom();
      var htmlEncode = require_html_encode();
      function renderString() {
        const carriageReturn = "\n";
        const indentation = "  ";
        let output = [];
        let pretty = false;
        let indents = 0;
        let stream = null;
        function writeToStream(message) {
          if (stream) {
            stream.write(message);
          }
        }
        function write(message) {
          output.push(message);
          writeToStream(message);
        }
        function writeAttributes(attributes) {
          if (attributes) {
            let key, value;
            const keys = Object.keys(attributes);
            const len = keys.length;
            const parts = [];
            for (var i = 0; i < len; ++i) {
              key = keys[i];
              value = attributes[key];
              if (key === constants.CLASS_NAME) {
                key = "class";
              } else if (key === "key") {
                key = constants.NOM_ATTR_KEY;
              } else if (key.indexOf("data") === 0) {
                key = camelToDash(key);
              }
              if (typeof value !== "function" && value !== null && value !== void 0 && value !== false) {
                write(` ${key}="${htmlEncode(value)}"`);
              }
            }
          }
        }
        function writeOpener(nodeName, attributes) {
          if (nodeName === "html") {
            write("<!DOCTYPE html>");
          }
          write(`<${nodeName}`);
          writeAttributes(attributes);
        }
        function writeCloser(nodeName) {
          write(`</${nodeName}>`);
        }
        function indent() {
          indents += 1;
        }
        function unindent() {
          indents -= 1;
        }
        function writeCarriageReturn() {
          if (pretty) {
            write(carriageReturn);
          }
        }
        function writeIndentation() {
          if (pretty) {
            for (let i = 0; i < indents; i += 1) {
              write(indentation);
            }
          }
        }
        function isCollapsible(nodeName, textValue, childCount) {
          return textValue === null && childCount === 0 && dom.collapsibleNodes.indexOf(nodeName) > -1;
        }
        function renderNode(element) {
          if (!element) {
            throw new Error("Cannot render falsy element.");
          }
          if (element.nodeName === "external") {
            return;
          }
          const attributes = element.attrs;
          const children = element.children;
          const nodeName = element.nodeName;
          const textValue = element.textValue;
          const unsafeContent = element.unsafeContent;
          writeIndentation();
          writeOpener(nodeName, attributes);
          const collapsed = isCollapsible(nodeName, textValue, children.length);
          if (!collapsed) {
            write(">");
          }
          if (nodeName === "style" && element.selectors && element.selectors.length > 0) {
            write(element.renderSelectors());
          } else if (textValue) {
            write(htmlEncode(textValue));
          } else if (unsafeContent) {
            write(unsafeContent);
          }
          if (children.length > 0) {
            writeCarriageReturn();
            indent();
            processChildren(children);
            unindent();
            writeIndentation();
          }
          if (collapsed) {
            write(" />");
            return;
          }
          writeCloser(nodeName);
          writeCarriageReturn();
        }
        function processChildren(children) {
          children.forEach((child) => {
            renderNode(child);
          });
        }
        return function render(element, optPrettyPrint, optStream) {
          pretty = optPrettyPrint;
          indents = 0;
          stream = optStream || null;
          renderNode(element);
          const result = output.join("");
          output = [];
          return result;
        };
      }
      module.exports = renderString;
    }
  });

  // src/svg.js
  var require_svg = __commonJS({
    "src/svg.js"(exports, module) {
      var builder = require_builder();
      var namespace = "http://www.w3.org/2000/svg";
      var svg = builder.elementWrapper("svg", namespace);
      var svgNodes = [
        "a",
        "altGlyph",
        "altGlyphDef",
        "altGlyphItem",
        "animate",
        "animateColor",
        "animateMotion",
        "animateTransform",
        "circle",
        "clipPath",
        "color-profile",
        "cursor",
        "defs",
        "desc",
        "ellipse",
        "feBlend",
        "feColorMatrix",
        "feComponentTransfer",
        "feComposite",
        "feConvolveMatrix",
        "feDiffuseLighting",
        "feDisplacementMap",
        "feDistantLight",
        "feFlood",
        "feFuncA",
        "feFuncB",
        "feFuncG",
        "feFuncR",
        "feGaussianBlur",
        "feImage",
        "feMerge",
        "feMergeNode",
        "feMorphology",
        "feOffset",
        "fePointLight",
        "feSpecularLighting",
        "feSpotLight",
        "feTile",
        "feTurbulence",
        "filter",
        "font",
        "font-face",
        "font-face-format",
        "font-face-name",
        "font-face-src",
        "font-face-uri",
        "foreignObject",
        "g",
        "glyph",
        "glyphRef",
        "hkern",
        "image",
        "line",
        "linearGradient",
        "marker",
        "mask",
        "metadata",
        "missing-glyph",
        "mpath",
        "path",
        "pattern",
        "polygon",
        "polyline",
        "radialGradient",
        "rect",
        "script",
        "set",
        "stop",
        "style",
        "svg",
        "switch",
        "symbol",
        "text",
        "textPath",
        "title",
        "tref",
        "tspan",
        "use",
        "view",
        "vkern"
      ];
      svgNodes.forEach((nodeName) => {
        svg[nodeName] = builder.elementWrapper(nodeName, namespace);
      });
      module.exports = svg;
    }
  });

  // index.js
  var require_nomplate = __commonJS({
    "index.js"(exports, module) {
      var Element = require_element();
      var builder = require_builder();
      var config = require_config();
      var constants = require_constants();
      var dom = require_dom();
      var events = require_events();
      var generateId = require_generate_id();
      var htmlEncode = require_html_encode();
      var operations = require_operations();
      var ready = require_ready();
      var renderElement = require_render_element();
      var renderString = require_render_string();
      var scheduler = require_scheduler();
      var svg = require_svg();
      var forceUpdate = builder.forceUpdate;
      module.exports = {
        Element,
        builder,
        config,
        constants,
        dom,
        events,
        forceUpdate,
        generateId,
        htmlEncode,
        operations,
        ready,
        renderElement,
        renderString,
        scheduler,
        svg
      };
    }
  });
  require_nomplate();
})();
